/**
--
-- RESOURCE_PROVIDERS
--
-- Stores a cross reference between resource provider IDs, which are used as
-- the primary keys for the resource provider entity beans. Also stores
-- reconciliation data for each resource provider.
--
-- PROVIDER_ID - Unique ID used as the primary key for the resource provider
--			   entity beans.
-- RESOURCE_DN - DN for the resource the resource provider is responsible for.
-- RECON_STATUS - Whether a reconciliation is currently running (0 = not reconciling,
--				1 = currently reconciling).
-- LAST_RECON_TIME - The amount of time the last reconcilation took.
-- MAX_RECON_DURATION - maximum duration allowed before timeout of the current
--					  reconcilation (minutes).
-- LOCK_SERVICE - Lock the service during reconciliation ( 1 - lock, 0 = do not lock)
-- REQUEST_ID - Tracks the process locking the service
-- CURRENT_REQUEST_COUNT - current number of executing requests
-- MAX_REQUEST_COUNT - maximum number of concurrent executing requests (or -1 = unlimited)
-- LAST_RESPONSE_TIME - timestamp of last response (to detect failed resources)
-- RESOURCE_STATUS - resource status (0 = ok, 1 = failed, 2 = failed service being tested)
--
*/
CREATE TABLE RESOURCE_PROVIDERS
  (
  PROVIDER_ID				   VARCHAR(20) NOT NULL,
  RESOURCE_DN				   VARCHAR(2000),
  RECON_STATUS				  NUMBER,
  LAST_RECON_TIME			   DATE,
  MAX_RECON_DURATION			NUMBER(20),
  LOCK_SERVICE			NUMBER,
  REQUEST_ID					VARCHAR(20),
  CURRENT_REQUEST_COUNT		NUMBER,
  MAX_REQUEST_COUNT		NUMBER,
  LAST_RESPONSE_TIME		DATE,
  SERVER VARCHAR2(20),
  RESOURCE_STATUS   NUMBER,
  RESTART_TIME          DATE,
CONSTRAINT RESOURCE_PROVIDERS_PK PRIMARY KEY(PROVIDER_ID) USING
	INDEX
	TABLESPACE enrole_indexes
  )
  TABLESPACE enrole_data;

/**
--
-- REMOTE_SERVICES_REQUESTS
--
-- Table of requests that where made in an asynchronous fasion, or where
-- made while a reconciliation was ongoing.
--
-- PROVIDER_ID - Unique ID used as the primary key for the resource provider
--			   entity beans.
-- REQUEST_ID - the ID of the request that was made.
-- TYPE - The type of request (GENERIC_REQUEST = 0, ASYNCHRONOUS_REQUEST = 1,
--							 INTRA_RECON_REQUEST = 2, SERVICE_DEFERRED_REQUEST = 3)
-- OPERATION - (NOOP = 0, ADD = 1, MODIFY = 2, DELETE = 3, SUSPEND = 4, RESTORE = 5, CHANGE_PASSWORD = 6)
-- REQUEST_TIME - The time the request was made.
-- EXPIRATION_TIME - The time the request expires. If null the request never expires. (Currently not used.)
-- TARGET - The owner of the account for add or the dn for other operations
-- SERVICE_DN - The distinguished name of the service instance in the directory
-- DATA - The data for the request (attribute values for add and modify).  Serialized Java Collection.
-- CONNECTION_POINT - The callback to complete the workflow process.  Serialized Java object.
--
*/
CREATE TABLE REMOTE_SERVICES_REQUESTS
  (
  PROVIDER_ID		   VARCHAR(20),
  REQUEST_ID			VARCHAR(20) NOT NULL,
  TYPE				  NUMBER,
  OPERATION			 NUMBER,
  REQUEST_TIME		  DATE,
  EXPIRATION_TIME	   DATE,
  TARGET				VARCHAR(2000),
  SERVICE_DN			VARCHAR(2000) NOT NULL,
  DATA			CLOB,
  CONNECTION_POINT BLOB,
CONSTRAINT REMOTE_SVR_REQ_FK FOREIGN KEY (PROVIDER_ID)
  REFERENCES RESOURCE_PROVIDERS (PROVIDER_ID),
CONSTRAINT REMOTE_SVR_REQ_PK PRIMARY KEY(REQUEST_ID)
	USING INDEX
	TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;

CREATE INDEX RS_REQ_PID_X ON REMOTE_SERVICES_REQUESTS(PROVIDER_ID)
  TABLESPACE enrole_indexes;

CREATE INDEX RS_REQ_RTIME_X ON REMOTE_SERVICES_REQUESTS(REQUEST_TIME)
  TABLESPACE enrole_indexes;


/**
--
-- REMOTE_RESOURCES_RECONS
--
-- Table of reconciliation units associates with a given resource provider.
--
-- PROVIDER_ID - Unique ID used as the primary key for the resource provider
--			   entity beans.
-- RECON_ID - Unique ID for each reconcialation unit.
-- DAY_OF_MONTH - Number indicating which day of the month the recon should be
--				run on.
-- MONTH_NUM - Number indicating which month the recon should be run on.
-- DAY_OF_WEEK - Number indicating which day of the week the recon should be run on.
-- DAY_OF_WEEK - Number indicating which hour of the day the recon should be run on.
-- MINUTE_NUM - Number indicating which minute of the hour the recon should be run on.
-- MAX_DURATION - The maximum allowed duration allowed before the recon will time out (minutes)
-- LOCK_SERVICE - Lock the service during reconciliation ( 1 - lock, 0 = do not lock)
--
*/
CREATE TABLE REMOTE_RESOURCES_RECONS
  (
  PROVIDER_ID		   VARCHAR(20) NOT NULL,
  RECON_ID			  NUMBER(20)  NOT NULL,
  DAY_OF_MONTH		  NUMBER,
  MONTH_NUM			 NUMBER,
  DAY_OF_WEEK		   NUMBER,
  HOUR_NUM			  NUMBER,
  MINUTE_NUM			NUMBER,
  MAX_DURATION		NUMBER(20),
  LOCK_SERVICE		  NUMBER DEFAULT 1,
  CONSTRAINT REMOTE_RES_RECONS_FK FOREIGN KEY(PROVIDER_ID)
	REFERENCES RESOURCE_PROVIDERS(PROVIDER_ID),
  CONSTRAINT REMOTE_RES_RECONS_PK PRIMARY KEY(PROVIDER_ID, RECON_ID)
	USING INDEX
	TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;

/**
--
-- REMOTE_RESOURCES_RECON_QUERIES
--
-- Table of reconciliation queries associates with a given reconciliation unit.
--
-- PROVIDER_ID - Unique ID used as the primary key for the resource provider
--			   entity beans.
-- RECON_ID - Unique ID for each reconcialation unit.
-- QUERY_ID - Unique ID for each reconcialation query.
-- RECON_FILTER - Filter associated with this reconciliation query.
-- RECON_BASE - Search base associated with this reconciliation query.
-- MAX_DURATION - not used
-- MAX_ENTRIES - not used
-- ATTRIBUTES - attributes to return in a reconciliation
--
*/
CREATE TABLE REMOTE_RESOURCES_RECON_QUERIES
  (
  PROVIDER_ID		   VARCHAR(20) NOT NULL,
  RECON_ID			  NUMBER(20)  NOT NULL,
  QUERY_ID			  NUMBER(20)  NOT NULL,
  RECON_FILTER		  VARCHAR2(4000),
  RECON_BASE			VARCHAR2(4000),
  MAX_DURATION		  NUMBER(20),
  MAX_ENTRIES		   NUMBER(20),
  ATTRIBUTES			CLOB,
CONSTRAINT REMOTE_RECON_QUERIES_FK FOREIGN KEY(PROVIDER_ID,
											   RECON_ID)
	REFERENCES REMOTE_RESOURCES_RECONS(PROVIDER_ID,
											  RECON_ID),
CONSTRAINT REMOTE_RECON_QUERIES_PK PRIMARY KEY(PROVIDER_ID,
											   RECON_ID,
											   QUERY_ID)
	USING
	INDEX
	TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;

/**
--
-- LCR_INPROGRESS_TABLE
--
-- Table containing information for Lifecycle Rule (LC Rule) initiated processes in progress.
--
-- TENANT		   non null tenant for this LC Rule (short name)
-- RULE_ID		  non null Lifecycle Rule ID (long)
-- RULE_OP		  LC Rule Operation (string)
-- CHILD_ID		process ID for the child process of this LC Rule
-- START_TIME	time the LC Rule child process started
-- ENTITY_ID	   non null entity this child process is operating on
--
*/
CREATE TABLE LCR_INPROGRESS_TABLE (
  TENANT VARCHAR(256) NOT NULL,
  RULE_ID NUMBER(20) NOT NULL,
  RULE_OP VARCHAR(256),
  CHILD_ID NUMBER(20),
  START_TIME NUMBER(20),
  ENTITY_ID VARCHAR(256) NOT NULL,
  CONSTRAINT Unique_RuleEntity PRIMARY KEY(RULE_ID,ENTITY_ID)
	USING
	INDEX
	TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;

/**
--
-- PO_TOPIC_TABLE
--
-- Table containing topic information for Post Office feature
--
-- TENANT				 non null tenant (short name)
-- TOPIC				  non null topic name (string)
-- SERVER				 identifies which cluster member is working on data
-- CHECKPOINT_TIME		identifies the time the cluster member started work
-- TOPIC_ID			   guuid that maps to TENANT above (non null, unique)
--
*/
CREATE TABLE PO_TOPIC_TABLE
  (
  TENANT						VARCHAR(256) NOT NULL,
  TOPIC						 VARCHAR(256) NOT NULL,
  SERVER						VARCHAR(20),
  CHECKPOINT_TIME			   NUMBER(20),
  TOPIC_ID					  NUMBER(20) NOT NULL,
  CONSTRAINT Unique_TenantTopic PRIMARY KEY(TENANT,
											TOPIC),
  CONSTRAINT Unique_TopicID UNIQUE(TOPIC_ID)
  USING
	INDEX
	TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;




/**
--
-- PO_NOTIFICATION_TABLE
--
-- Table containing individual notification information for Post Office feature
--
-- NOTIFICATION_ID	guid that identifies this notification (non null, unique)
-- TOPIC_ID			  guid that maps to TENANT above (non null, unique)
-- SUBJECT			  subject of the notification
-- TEXTBODY			  text body of the notification
-- RECEIVE_TIME		non null time in millis this notification was received
-- RECIPIENT_EADDR	non null email address for this notification
-- RECIPIENT_LOCALE	locale for this recipient (ll_CC format)
*/
CREATE TABLE PO_NOTIFICATION_TABLE
  (
  NOTIFICATION_ID		NUMBER(20) NOT NULL,
  TOPIC_ID				NUMBER(20) NOT NULL,
  SUBJECT				VARCHAR(2000),
  TEXTBODY				Long,
  RECEIVE_TIME			NUMBER(20) NOT NULL,
  RECIPIENT_EADDR		VARCHAR(320) NOT NULL,
  RECIPIENT_LOCALE		VARCHAR(256),
  CONSTRAINT TenantTopic_parent FOREIGN KEY(TOPIC_ID) REFERENCES PO_TOPIC_TABLE(TOPIC_ID),
  CONSTRAINT NOTIFICATION_ID_PK PRIMARY KEY(NOTIFICATION_ID)
  USING
	INDEX
	TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;


CREATE INDEX PO_NotifyIndex ON PO_NOTIFICATION_TABLE(TOPIC_ID)
  TABLESPACE enrole_indexes;


/**
--
-- PO_NOTIFICATION_HTMLBODY_TABLE
--
-- Table containing XHTML Body for an individual notification (matches to PO_NOTIFICATION_TABLE)
--
-- NOTIFICATION_ID	guid that identifies this notification (non null, unique)
-- HTMLBODY			  html body of the notification
*/

CREATE TABLE PO_NOTIFICATION_HTMLBODY_TABLE (
  NOTIFICATION_ID			   NUMBER(20) NOT NULL,
  HTMLBODY					  Long,
  CONSTRAINT NotificationIDLinkage FOREIGN KEY(NOTIFICATION_ID) REFERENCES PO_NOTIFICATION_TABLE(NOTIFICATION_ID) ON DELETE CASCADE,
  CONSTRAINT NOTIFICATION_ID_HTML_PK PRIMARY KEY(NOTIFICATION_ID)
  USING
	INDEX
	TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;



/**
--
-- SCHEDULED_MESSAGE
--
-- Table of scheduled one time or re-occuring events. Usually it contains scheduled user requests submitted
-- by the workflow engine or re-ocurring reconcialation events.
--
-- SCHEDULED_TIME - The long interger that represents the time of the scheduled event, which is
--				  the number of milliseconds since January 1, 1970, 00:00:00 GMT
--
-- SCHEDULED_MESSAGE_ID - Unique ID for each scheduled event
--
-- MESSAGE - A serialized object that represents the detail information of the scheduled event
--
-- SERVER - The server that picks up the scheduled event most recently
--
-- CHECKPOINT_TIME - The long interger that represents the last pick up time of the scheduled event, which
--				   is the number of milliseconds since January 1, 1970, 00:00:00 GMT
--
-- REFERENCE_ID - The is only used for scheduled workflow events, it is the workflow process ID that
--				sheduled event is coming from.
--
-- REFERENCE2_ID - Generic reference field.
--
*/
CREATE TABLE SCHEDULED_MESSAGE (
  SCHEDULED_TIME						NUMBER(20),
  SCHEDULED_MESSAGE_ID				  NUMBER(20) NOT NULL,
  MESSAGE							   Long,
  SERVER								VARCHAR2(20),
  CHECKPOINT_TIME					   NUMBER(20),
  REFERENCE_ID						  NUMBER(20),
  REFERENCE2_ID						 VARCHAR2(50),
CONSTRAINT SCHEDULED_MESSAGE_PK PRIMARY KEY(
									   SCHEDULED_MESSAGE_ID)
	USING
	INDEX
	TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;

CREATE INDEX SCHEDULED_MESSAGE_TIME_X ON SCHEDULED_MESSAGE (SCHEDULED_TIME, SERVER)
  TABLESPACE enrole_indexes;

CREATE INDEX SCHEDULED_MESSAGE_CHECKPOINT_X ON SCHEDULED_MESSAGE (CHECKPOINT_TIME)
  TABLESPACE enrole_indexes;

CREATE INDEX SCHEDULED_MESSAGE_REFERENCE_X ON SCHEDULED_MESSAGE (REFERENCE_ID)
  TABLESPACE enrole_indexes;

CREATE INDEX SCHEDULED_MSG_X ON SCHEDULED_MESSAGE (REFERENCE2_ID)
  TABLESPACE enrole_indexes;
/**
--
-- NEXTVALUE
-- Table to keep the current maximum value of primary key integer. It only contains
-- one record, for each identfier
--
-- ID -- The next primary key id to be used
--
*/
CREATE TABLE NEXTVALUE
	(
	ID				  VARCHAR2(50) NOT NULL,
	NEXTID			  NUMBER(10) NOT NULL,
CONSTRAINT NEXTVALUE_PK PRIMARY KEY(ID) USING
	 INDEX
	 TABLESPACE enrole_indexes
	)
  TABLESPACE enrole_data;


COMMIT;
/**
--
-- PROCESS
--
-- This table stores all pending, running and historical requests submitted
-- to the enrole workflow system. Each request is represented as a process.
--
-- ID - The process id
-- PARENT_ID - The parent process id if there is any
-- NAME - The process name
-- TYPE - The process type
-- DEFINITION_ID - The process definition ID
-- REQUESTER_TYPE - The requester type
-- REQUESTER  - The requester identity
-- REQUESTER_NAME - The requester name
-- DESCRIPTION - The process description
-- PRIORITY - The process priority
-- SCHEDULED - The scheduled starting time of the process
-- STARTED - The starting time of the process
-- COMPLETED - The completed time of the process
-- LASTMODIFIED - The last modified time of the process
-- SUBMITTED - The time the process is submitted
-- STATE - The process state
-- COMMENTS - The comments for the process
-- RESULT_SUMMARY - The result summary code of the process
-- RESULT_DETAIL - The result detail information of the process
-- TENANT - The DN of the tenant the requester belongs to
--
*/
CREATE TABLE PROCESS
	  (
				  ROOT_PROCESS_ID	   NUMBER(20) NOT NULL,
				ID					  NUMBER(20) NOT NULL,
				PARENT_ID			   NUMBER(20),
				PARENT_ACTIVITY_ID	  NUMBER(20),
				NAME					varchar2(100),
				TYPE					varchar2(2) NOT NULL,
				DEFINITION_ID		   varchar2(2000) NOT NULL,
				REQUESTER_TYPE		  varchar2(2),
				REQUESTER			   varchar2(2000),
				REQUESTER_NAME		  varchar2(100),
				DESCRIPTION			 varchar2(300),
				PRIORITY				NUMBER(5),
				SCHEDULED			   varchar2(50),
				STARTED				 varchar2(50),
				COMPLETED			   varchar2(50),
				LASTMODIFIED			varchar2(50),
				SUBMITTED			   VARCHAR2(50),
				STATE				   varchar2(1),
				NOTIFY				  NUMBER(10),
				REQUESTEE			   varchar2(2000),
				REQUESTEE_NAME		  varchar2(100),
				SUBJECT				 varchar2(2000),
				SUBJECT_PROFILE		 varchar2(100),
				SUBJECT_SERVICE		 varchar2(100),
				COMMENTS				varchar2(200),
				RESULT_SUMMARY		  varchar2(2),
				RESULT_DETAIL		   Long,
				TENANT				  varchar2(2000),
CONSTRAINT PROCESS_PK PRIMARY KEY(ID) USING
	INDEX
	TABLESPACE enrole_indexes
	)
  TABLESPACE enrole_data;

CREATE INDEX PROC_STATE_X ON PROCESS(STATE)
  TABLESPACE enrole_indexes;

CREATE INDEX PROCESS_VPC1_X ON PROCESS(PARENT_ID, SUBMITTED, ID, STATE)
  TABLESPACE enrole_indexes;

CREATE INDEX PROCESS_PARENT_X ON PROCESS(PARENT_ID) TABLESPACE enrole_indexes;

/**
--
-- ACTIVITY
--
-- This table contains records to keep track of each process's execution status.
--
-- ID - The activity id
-- PROCESS_ID - The process id that the activity belongs to
-- DEFINITION_ID - The activity definition ID
-- ACTIVITY_INDEX - The index of the activity if it is inside a loop
-- SUBPROCESS_ID - The ID of the subprocess associated with the activity
-- NAME - The activity name
-- DESCRIPTION - The activity description
-- TYPE - The activity type
-- SUBTYPE - The activity subtype
-- PRIORITY - The activity priority (Currently not used)
-- STARTED - The time the activity started
-- COMPLETED - The time the activity completed
-- LASTMODIFIED - The time the activity was last modified
-- STATE - The activity state
-- RESULT_SUMMARY - The result summary code for the activity
-- RESULT_DETAIL - The result detail for the activity
--
*/
CREATE TABLE ACTIVITY (
				ID					  NUMBER(20) NOT NULL,
				PROCESS_ID			  NUMBER(20) NOT NULL,
				DEFINITION_ID		   varchar2(100) NOT NULL,
				ACTIVITY_INDEX		  NUMBER(10),
				LOOP_COUNT			  NUMBER(10),
				LOOP_RUNCOUNT		   NUMBER(10),
				RETRY_COUNT			 NUMBER(10),
				LOCK_COUNT		NUMBER(10),
				SUBPROCESS_ID		   NUMBER(20),
				NAME					varchar2(100),
				DESCRIPTION			 varchar2(300),
				TYPE					varchar2(1),
				SUBTYPE				 varchar2(2),
				PRIORITY				NUMBER(5),
				STARTED				 varchar2(50),
				COMPLETED			   varchar2(50),
				LASTMODIFIED			varchar2(50),
				STATE				   varchar2(1),
				RESULT_SUMMARY		  varchar2(5),
				RESULT_DETAIL		   Long,
CONSTRAINT ACTIVITY_FK FOREIGN KEY(PROCESS_ID)
	REFERENCES PROCESS(ID),
CONSTRAINT ACTIVITY_PK PRIMARY KEY(ID) USING
	INDEX
	TABLESPACE enrole_indexes
	)
  TABLESPACE enrole_data;

CREATE UNIQUE INDEX ACTIVITY_NATURAL_KEY ON ACTIVITY (PROCESS_ID, DEFINITION_ID, ACTIVITY_INDEX) TABLESPACE enrole_indexes;

/**
--
-- PROCESSDATA
--
-- This table keeps the runtime process data of a process
-- After process completion, this table will be removed
--
-- PROCESS_ID - The process ID associated with the data
-- DEF_ID - The definition ID for the corresponding relevant data in process definition
-- ACTIVITY_ID - The activity ID associated with the data if there is any
-- NAME - The data name
-- DESCRIPTION - The data description
-- TYPE - Data type
-- VALUE - Data value
--
*/
CREATE TABLE PROCESSDATA
(
	PROCESS_ID			  NUMBER(20) NOT NULL,
	DEF_ID				  varchar2(100),
	NAME					varchar2(100),
	CONTEXT				 varchar2(100),
	DESCRIPTION			 varchar2(300),
	TYPE					varchar2(500),
	COLLECTION_TYPE		 varchar2(500),
	VALUE				   Long,
	SMALL_VALUE			 varchar2(4000),
	VALUE_LAST_MODIFIED   NUMBER(20),
CONSTRAINT PROCESSDATA_FK FOREIGN KEY(PROCESS_ID)
	REFERENCES PROCESS(ID),
CONSTRAINT PROCESSDATA_PK PRIMARY KEY(PROCESS_ID, DEF_ID) USING
	INDEX
	TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;
/**
--
-- PROCESSLOG
--
-- This table keeps records of audit events associated with a process
--
-- ID - The processlog ID
-- PROCESS_ID - The ID of the process associated
-- ACTIVITY_ID - The ID of the activity associated
-- CREATED - Time the log record created
-- EVENTTYPE - Log event type
-- OLD_PARTICIPANT_TYPE - Old participant type for assignment change event
-- OLD_PARTICIPANT_ID - Old participant ID for assignment change event
-- NEW_PARTICIPANT_TYPE - New participant type for assignment change event
-- NEW_PARTICIPANT_ID - New participant ID for assignment change event
-- REQUESTOR_TYPE - The requester type for any user related event
-- REQUESTOR - The requester ID for any user related event
-- OLD_STATE - The old state for state change event
-- NEW_STATE - The new state for state change event
-- DATA_ID - The data id for data change event
-- NEW_DATA - The data value for data change event
--
*/
CREATE TABLE PROCESSLOG
   (
				ID					  NUMBER(20) NOT NULL,
				PROCESS_ID			  NUMBER(20),
				ACTIVITY_ID			 NUMBER(20),
				CREATED				 varchar2(50),
				EVENTTYPE			   varchar2(2) NOT NULL,
				OLD_PARTICIPANT_TYPE	varchar2(2),
				OLD_PARTICIPANT_ID	  varchar2(2000),
				NEW_PARTICIPANT_TYPE	varchar2(2),
				NEW_PARTICIPANT_ID	  varchar2(2000),
				REQUESTOR_TYPE		  varchar2(2),
				REQUESTOR			   varchar2(2000),
				OLD_STATE			   varchar2(1),
				NEW_STATE			   varchar2(1),
				DATA_ID				 varchar2(100),
				NEW_DATA				Long,
CONSTRAINT PROCESSLOG_FK FOREIGN KEY(PROCESS_ID)
	REFERENCES PROCESS(ID),
CONSTRAINT PROCESSLOG_PK PRIMARY KEY(ID) USING
	INDEX
	TABLESPACE enrole_indexes
	)
  TABLESPACE enrole_data;
/**
--
-- WORKITEM
--
-- This table keeps records of workitems associated with manual workflow activities
-- for running process. The records associated with a process will be removed after
-- process completion
--
-- ID  -  The workitem ID
-- PROCESS_ID - The process ID associated with the workitem
-- ACTIVITY_ID - The activity ID assciated with the workitem
-- PARTICIPANT_TYPE - The workitem participant type
-- PARTICIPANT  -  The workitem participant identity
-- CREATED - The time the workitem created
--
*/
CREATE TABLE WORKITEM
  (
		ID					  NUMBER(20) NOT NULL,
		PROCESS_ID			  NUMBER(20) NOT NULL,
		ACTIVITY_ID			 NUMBER(20) NOT NULL,
		DUE_DATE				NUMBER(20),
		LOCK_OWNER			  varchar2(512),
		DESCRIPTION			 varchar2(4000),
		PROCESS_DEFINITION_ID   varchar2(512) NOT NULL,
		ACTIVITY_DEFINITION_ID  varchar2(100) NOT NULL,
		CREATED				 varchar2(50),
		INPUT_PARAMETERS		Long,
		ACTIVITY_TYPE VARCHAR2(1),
		ACTIVITY_SUBTYPE VARCHAR2(2),
		ACTIVITY_NAME VARCHAR2(100),
		REQUESTEE_NAME VARCHAR2(100),
		REQUESTER_NAME VARCHAR2(100),
		SUBJECT VARCHAR2(2000),

CONSTRAINT WORKITEM2_FK FOREIGN KEY(ACTIVITY_ID)
	REFERENCES ACTIVITY(ID),
CONSTRAINT WORKITEM_FK FOREIGN KEY(PROCESS_ID)
	REFERENCES PROCESS(ID),
CONSTRAINT WORKITEM_PK PRIMARY KEY(ID) USING
	INDEX
	TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;

CREATE INDEX WORKITEM_PID_X ON WORKITEM (PROCESS_ID)
TABLESPACE enrole_indexes;
CREATE INDEX WORKITEM_AID_X ON WORKITEM (ACTIVITY_ID)
TABLESPACE enrole_indexes;
CREATE INDEX WORKITEM_ADEF_X ON WORKITEM (ACTIVITY_DEFINITION_ID)
TABLESPACE enrole_indexes;
CREATE INDEX WORKITEM_PDEF_X ON WORKITEM (PROCESS_DEFINITION_ID)
TABLESPACE enrole_indexes;


/**
--
-- WI_PARTICIPANT
--
-- This table keeps the participants associated with manual workflow activities.
-- ID  -  The participant id
-- WORKITEM_ID - The work item ID associated with the participant
-- PARTICIPANT_TYPE - The workitem participant type
-- PARTICIPANT  -  The workitem participant identity
--
*/
CREATE TABLE WI_PARTICIPANT
  (
		ID					  NUMBER(20) NOT NULL,
		WORKITEM_ID			  NUMBER(20) NOT NULL,
		PARTICIPANT_TYPE		varchar2(2) NOT NULL,
		PARTICIPANT			 varchar2(512) NOT NULL,
CONSTRAINT WORKITEM_ID_FK FOREIGN KEY(WORKITEM_ID)
	REFERENCES WORKITEM(ID) ON DELETE CASCADE,
CONSTRAINT PARTICIPANT_PK PRIMARY KEY(ID) USING
	INDEX
	TABLESPACE enrole_indexes
   )
  TABLESPACE enrole_data;

CREATE INDEX WIPART_WIID_X ON WI_PARTICIPANT (WORKITEM_ID)
TABLESPACE enrole_indexes;
CREATE INDEX WIPART_DN_X ON WI_PARTICIPANT (PARTICIPANT)
TABLESPACE enrole_indexes;

CREATE TABLE SYNCH_POINT (
  PROCESS_ID NUMBER(20) NOT NULL,
  DEFINITION_ID varchar2(100) NOT NULL,
  ACTIVITY_INDEX NUMBER(10) NOT NULL,
  WAIT_LOCK NUMBER(10) NOT NULL,
  JOIN_ENABLED NUMBER NOT NULL,
  CONSTRAINT SYNCH_POINT_PK PRIMARY KEY(PROCESS_ID, DEFINITION_ID, ACTIVITY_INDEX) USING
  INDEX
  TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;

CREATE INDEX SYNCH_POINT_PID_X ON SYNCH_POINT (PROCESS_ID DESC)
TABLESPACE enrole_indexes;

CREATE TABLE PASSWORD_TRANSACTION
  (
		TRANSACTION_ID		  NUMBER(20) NOT NULL,
		ACCOUNT_DN					  VARCHAR2(2000),
		CREATION_DATE		   VARCHAR2(50),
		PROCESS_ID					  NUMBER(20) NOT NULL,
		ACTIVITY_ID					 NUMBER(20) NOT NULL,
		PASSWORD						VARCHAR2(500),
CONSTRAINT PASSWORD_TRANSACTION_PK
PRIMARY KEY(TRANSACTION_ID) USING
				INDEX
				TABLESPACE enrole_indexes
  )
TABLESPACE enrole_data;

/**
--
-- PENDING
--
-- This table keeps the pending provisioning processes
--
-- PROCESS_ID - The process ID associated with the data
-- PERSON_DN  - The definition ID for the corresponding relevant data in process definition
-- SERVICE_DN - The activity ID associated with the data if there is any
--
*/
CREATE TABLE PENDING
(
	PROCESS_ID			  NUMBER(20) NOT NULL,
	PERSON_DN			   varchar2(2000),
	SERVICE_DN			  varchar2(2000),
CONSTRAINT PENDING_FK FOREIGN KEY(PROCESS_ID)
	REFERENCES PROCESS(ID),
CONSTRAINT PENDING_PK PRIMARY KEY(PROCESS_ID) USING
	INDEX
	TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;

/**
--
-- LISTDATA
--
-- This table is used to persist large data list for performance optimazation
--
-- DATA_ID - The unique identifier of the data
-- INDEX   - The index of the list element
-- VALUE   - The serialized list element
--
*/
CREATE TABLE LISTDATA
(
	DATA_ID  NUMBER(20) NOT NULL,
	INDEX_ID NUMBER(10) NOT NULL,
	VALUE	Long,
CONSTRAINT LISTDATA_PK PRIMARY KEY(DATA_ID, INDEX_ID) USING
	INDEX
	TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;

CREATE INDEX PENDING_FOR_PERSON ON PENDING (PERSON_DN) TABLESPACE enrole_indexes;

CREATE TABLE REPORT (
	ID INT NOT NULL,
	TITLE VARCHAR(255) NOT NULL,
	TYPE VARCHAR(255) NOT NULL,
	AUTHOR VARCHAR(255),
	REPORT_SIZE INT,
	REPORT_DATA LONG RAW,
	STYLESHEET_NAME VARCHAR(255),
	EDITABLE CHAR(1),
	REPORTCATEGORY VARCHAR(255) NOT NULL,
	REPORTSUBTYPE VARCHAR(255) NOT NULL,
	CONSTRAINT REPORT_P PRIMARY KEY (ID)
)
TABLESPACE enrole_data;

CREATE TABLE ENTITY_COLUMN (
	ENTITY_NAME VARCHAR(255) NOT NULL,
	COLUMN_NAME VARCHAR(255) NOT NULL,
	ATTRIBUTE_NAME VARCHAR(255) NOT NULL,
	MULTI_VALUED CHAR(1),
	IMPLICITLY_MAPPED CHAR(1),
	AVAILABLE_FOR_REPORTING VARCHAR(255),
	TABLE_NAME VARCHAR(255),
	CONSTRAINT ENTITY_COLUMNP PRIMARY KEY (ENTITY_NAME, ATTRIBUTE_NAME)
)
TABLESPACE enrole_data;


CREATE TABLE COLUMN_REPORT (
	ENTITY_NAME VARCHAR(255) NOT NULL,
	COLUMN_NAME VARCHAR(255) NOT NULL,
	REPORT_ID INT NOT NULL,
	CONSTRAINT COLUMN_REPORT_P PRIMARY KEY (REPORT_ID, ENTITY_NAME, COLUMN_NAME)
)
TABLESPACE enrole_data;

CREATE TABLE AUTHORIZATION_OWNERS (
	  USERDN VARCHAR(255) NOT NULL,
	 CONTAINERDN VARCHAR(255) NOT NULL,
	 PRIMARY KEY(USERDN, CONTAINERDN)
)
TABLESPACE enrole_data;


CREATE TABLE ACI (
	DN VARCHAR(255) NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	SCOPE VARCHAR(255),
	TARGET VARCHAR(255) NOT NULL,
	PARENT VARCHAR(255),
	CATEGORY VARCHAR(255),
	ObjectFilter VARCHAR(1023),
	PRIMARY KEY(DN, NAME, TARGET)
)
TABLESPACE enrole_data;


CREATE TABLE ACI_ROLEDNS (
	DN VARCHAR(255) NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	TARGET VARCHAR(255) NOT NULL,
	ROLEDN VARCHAR(255) NOT NULL
)
TABLESPACE enrole_data;


CREATE TABLE ACI_PRINCIPALS (
	DN VARCHAR(255) NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	TARGET VARCHAR(255) NOT NULL,
	PRINCIPALNAME VARCHAR(255) NOT NULL
)
TABLESPACE enrole_data;


CREATE TABLE ACI_PERMISSION_ATTRIBUTERIGHT (
	DN VARCHAR(255) NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	TARGET VARCHAR(255) NOT NULL,
	ACTION VARCHAR(6) NOT NULL,
	OPERATION VARCHAR(3) NOT NULL,
	ATTRIBUTERIGHT VARCHAR(255) NOT NULL
)
TABLESPACE enrole_data;


CREATE TABLE ACI_PERMISSION_CLASSRIGHT (
	DN VARCHAR(255) NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	TARGET VARCHAR(255) NOT NULL,
	ACTION VARCHAR(6) NOT NULL,
	CLASSRIGHT VARCHAR(255) NOT NULL
)
TABLESPACE enrole_data;


CREATE TABLE ENTITLEMENT (
	DN VARCHAR(255) NOT NULL,
	TYPE VARCHAR(255) NOT NULL,
	SERVICETARGETTYPE VARCHAR(255),
	SERVICETARGETNAME VARCHAR(255),
	PROCESSDN VARCHAR(255)
)
TABLESPACE enrole_data;

CREATE TABLE ENTITLEMENT_PROVISIONINGPARAMS (
	DN VARCHAR(255) NOT NULL,
	ATTRIBUTEVALUE VARCHAR(4000) NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	ENFORCEMENT VARCHAR(255),
	EXPRTYPE VARCHAR(255)
)
TABLESPACE enrole_data;

CREATE TABLE SYNCHRONIZATION_HISTORY (
	SYNC_ID number(19,0)  NOT NULL ,
	REQUESTOR VARCHAR (255) NOT NULL,
	REQ_TYPE VARCHAR (255),
	REQ_NAME VARCHAR (255),
	STATUS VARCHAR (255),
	TENANT VARCHAR (255),
	STATUS_DETAIL VARCHAR (255),
	SCHEDULED_TIME number(19,0),
	SUBMITTED_TIME number(19,0),
	STARTED_TIME number(19,0) NOT NULL,
	COMPLETED_TIME number(19,0),
	SERVER_NAME VARCHAR (255),
	PRIMARY KEY (SYNC_ID, STARTED_TIME)
)
TABLESPACE enrole_data;

CREATE TABLE SYNCHRONIZATION_LOCK (
	HOST VARCHAR (255) NOT NULL,
	PRIMARY KEY (HOST)
)
TABLESPACE enrole_data;

CREATE TABLE ENROLE.RESOURCES_SYNCHRONIZATIONS (
	SYNC_ID number(19,0)  NOT NULL,
	DAY_OF_MONTH number(10)  NOT NULL,
	MONTH_NUM number(10) NOT NULL,
	DAY_OF_WEEK number(10) NOT NULL,
	HOUR_NUM number(10) NOT NULL,
	MINUTE_NUM number(10) NOT NULL,
	MAX_DURATION number(19,0),
	PRIMARY KEY (SYNC_ID)
)
TABLESPACE enrole_data;

CREATE TABLE CHANGELOG (
	CHANGE_NUMBER number(10) NOT NULL
)
TABLESPACE enrole_data;

CREATE TABLE RECONCILIATION (
	RECONID VARCHAR(255) NOT NULL,
	SERVICEDN VARCHAR(2000) NOT NULL,
	PROCESSEDACCOUNTS INTEGER NOT NULL,
	LOCALACCOUNTS INTEGER NOT NULL,
	TIMUSERACCOUNTS INTEGER NOT NULL,
	POLICYVIOLATIONS INTEGER NOT NULL,
	STARTED DATE NOT NULL,
	COMPLETED DATE NOT NULL,
	PRIMARY KEY (RECONID)
)
TABLESPACE enrole_data;

CREATE TABLE RECONCILIATION_INFO (
	RECONID VARCHAR(255) NOT NULL,
	ACCOUNTID VARCHAR(255),
	POLICYCOMPLIANCESTATUS VARCHAR(20),
	USERNAME VARCHAR(255),
	OPERATION VARCHAR(20),
	REMARKS VARCHAR(1000),
	FOREIGN KEY (RECONID) REFERENCES RECONCILIATION (RECONID)
)
TABLESPACE enrole_data;

/**
--
-- AUTH_KEY
--
-- This table is used to store the keys for signing and verifying authentication requests.
--
-- Y   - The public key in the DSA algorithm
-- P   - The prime number in the DSA algorithm
-- Q   - The sub-prime number in the DSA algorithm
-- G   - The modulus in the DSA algorithm
-- X   - The private key in the DSA algorithm
--
*/
CREATE TABLE AUTH_KEY (
  Y VARCHAR(2000) NOT NULL,
  P VARCHAR(2000) NOT NULL,
  Q VARCHAR(2000) NOT NULL,
  G VARCHAR(2000) NOT NULL,
  X VARCHAR(2000) NOT NULL
)
TABLESPACE enrole_data;

/**
--
-- WORKFLOW_CALLBACK
--
-- This table is used to store information for the workflow callback API.
--
-- ID			  - The primary key
-- PROCESS_ID	  - The process identifier this callback is associated
-- MESSAGE_OBJECT  - The callback message object.
-- EVENT_TRIGGER   - The process state the callback should be called.
--
*/
CREATE TABLE WORKFLOW_CALLBACK (
  ID NUMBER(20) NOT NULL,
  PROCESS_ID NUMBER(20) NOT NULL,
  MESSAGE_OBJECT VARCHAR(2000) NOT NULL,
  EVENT_TRIGGER  CHAR(1),
  CONSTRAINT WF_CALLBACK_FK FOREIGN KEY(PROCESS_ID) REFERENCES PROCESS(ID),
  CONSTRAINT WF_CALLBACK_PK PRIMARY KEY(ID) USING
	INDEX
	TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;

/**
--
-- POLICY_ANALYSIS
--
-- This table is used to store policy analysis session information
--
--
*/
CREATE TABLE POLICY_ANALYSIS (
  ANALYSIS_ID	VARCHAR(32) NOT NULL,
  TENANT_NAME VARCHAR(64),
  STATUS NUMBER(2) NOT NULL,
  REASON NUMBER(2) NOT NULL,
  CONTEXT NUMBER(2) NOT NULL,
  CHANGE_TYPE NUMBER(2) NOT NULL,
  LAST_ACCESSED DATE NOT NULL,
  WORKERS_STARTED NUMBER(10) DEFAULT 0 NOT NULL,
  WORKERS_COMPLETED NUMBER(10) DEFAULT 0 NOT NULL,
  WORKERS_TOTAL NUMBER(10) DEFAULT 0 NOT NULL,
  ACCOUNTS_EVALUATED NUMBER(10) DEFAULT 0 NOT NULL,
  CONSTRAINT PK_POLICY_ANALYSIS PRIMARY KEY(ANALYSIS_ID)
	USING INDEX
	TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;

/**
--
-- ACCT_CHANGE, ATTR_CHANGE, POLICY_ANALYSIS_ERROR are used to stage policy change preview data
--
*/
CREATE TABLE ACCT_CHANGE (
  CHANGE_ID VARCHAR(32) NOT NULL,
  ACCT_UID VARCHAR(60) NOT NULL,
  ACCT_IDENTIFIER VARCHAR(255) NOT NULL,
  SERVICE_NAME VARCHAR(200) NOT NULL,
  SERVICE_IDENTIFIER VARCHAR(255) NOT NULL,
  OWNER_NAME VARCHAR(200) NOT NULL,
  OWNER_IDENTIFIER VARCHAR(255) NOT NULL,
  OPERATION_TYPE NUMBER(2) NOT NULL,
  PROVISION_PRIORITY NUMBER(5),
  SEQUENCE_NR NUMBER(5),
  REASON NUMBER(2) NOT NULL,
  REVOKE_CHANGE NUMBER(2) NOT NULL,
  STATUS NUMBER(2),
  POLICY_ANALYSIS_ID VARCHAR(32) NOT NULL,
  CONSTRAINT FK_POLICY_ANALYSIS FOREIGN KEY(POLICY_ANALYSIS_ID) REFERENCES POLICY_ANALYSIS(ANALYSIS_ID)
	ON DELETE CASCADE,
	CONSTRAINT PK_ACCT_CHANGE PRIMARY KEY(POLICY_ANALYSIS_ID, CHANGE_ID)
	USING INDEX
	TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;

CREATE TABLE ATTR_CHANGE (
  CHANGE_ID VARCHAR(32) NOT NULL,
  ATTR_NAME VARCHAR(100) NOT NULL,
  ATTR_VALUE VARCHAR(2000),
  OPERATION_TYPE NUMBER(2) NOT NULL,
  PRIVILEGE_ACTION_TYPE NUMBER(2) NOT NULL,
  ATTR_VALUE_PRESENCE NUMBER(2) NOT NULL,
  POLICY_ANALYSIS_ID VARCHAR(32) NOT NULL,
  ACCT_CHANGE_ID VARCHAR(32) NOT NULL,
  CONSTRAINT FK_ATTR_CHANGE FOREIGN KEY(POLICY_ANALYSIS_ID, ACCT_CHANGE_ID)
	REFERENCES ACCT_CHANGE(POLICY_ANALYSIS_ID, CHANGE_ID)
	ON DELETE CASCADE,
	CONSTRAINT PK_ATTR_CHANGE PRIMARY KEY(POLICY_ANALYSIS_ID, ACCT_CHANGE_ID, CHANGE_ID)
	USING INDEX
	TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;

CREATE TABLE POLICY_ANALYSIS_ERROR (
  ERROR_ID	VARCHAR(32) NOT NULL,
  ENTITY_NAME VARCHAR(100),
  ENTITY_IDENTIFIER VARCHAR(255),
  ENTITY_TYPE  NUMBER(2),
  SERVICE_NAME VARCHAR(200),
  SERVICE_IDENTIFIER VARCHAR(255),
  PERSON_NAME  VARCHAR(200),
  PERSON_IDENTIFIER VARCHAR(255),
  POLICY_NAME VARCHAR(100),
  POLICY_IDENTIFIER	VARCHAR(255),
  ATTR_NAME VARCHAR(100),
  ERROR_TYPE NUMBER(5) NOT NULL,
  ENTITY_ERROR_TYPE  NUMBER(5),
  ERROR_MESSAGE LONG NOT NULL,
  POLICY_ANALYSIS_ID VARCHAR(32) NOT NULL,
  CONSTRAINT FK_ANALYSIS_ERROR FOREIGN KEY(POLICY_ANALYSIS_ID) REFERENCES POLICY_ANALYSIS(ANALYSIS_ID)
	ON DELETE CASCADE,
  CONSTRAINT PK_ANALYSIS_ERROR PRIMARY KEY(ERROR_ID)
	USING INDEX
	TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;


CREATE TABLE COMPLIANCE_ALERT (
  CA_PROC_ID	 NUMBER(20),
  CA_ISSUE_DN	VARCHAR(512) NOT NULL,
  ACTIVITY_ID	  NUMBER(20),
  ACCOUNT_DN	VARCHAR(512) NOT NULL,
  PARTICIPANT_DN  VARCHAR(512),
  STARTED		CHAR(1),
  CONSTRAINT PK_CA PRIMARY KEY(CA_ISSUE_DN)
	USING INDEX
	TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;
CREATE INDEX CA_PID_X ON COMPLIANCE_ALERT(CA_PROC_ID)
TABLESPACE enrole_indexes;
CREATE INDEX CA_ACCDN_X ON COMPLIANCE_ALERT(ACCOUNT_DN)
TABLESPACE enrole_indexes;
CREATE INDEX CA_ACTID_X ON COMPLIANCE_ALERT(ACTIVITY_ID)
TABLESPACE enrole_indexes;


/**
These are the SQL statements for inserting values in the database for reporting purposes. All these columns are used by
out-of-box reports.
*/
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Account','DN','DN', 'N','N','table_dropped','Account');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Account','eraccountcompliance','eraccountcompliance','N','N','table_dropped','Account');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Account','eraccountstatus','eraccountstatus','N','N','table_dropped','Account');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Account','erlastaccessdate','erlastaccessdate','N','N','table_dropped','Account');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Account','erparent','erparent','N','N','table_dropped','Account');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Account','erservice','erservice','N','N','table_dropped','Account');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Account','eruid','eruid','N','N','table_dropped','Account');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Account','owner','owner','Y','N','table_dropped','Account_owner');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Role','DN','DN','N','N','table_dropped','Role');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Role','errolename','errolename','N','N','table_dropped','Role');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('OrganizationalContainer','description','description','Y','N','table_dropped','OrganizationalCont_description');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('OrganizationalContainer','DN','DN','N','N','table_dropped','OrganizationalCont0');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('OrganizationalContainer','erparent','erparent','N','N','table_dropped','OrganizationalCont0');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('OrganizationalContainer','name','name','N','N','table_dropped','OrganizationalCont0');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Person','cn','cn','Y','N','table_dropped','Person_cn');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Person','erparent','erparent','N','N','table_dropped','Person');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Person','DN','DN','N','N','table_dropped','Person');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Person','erpersonstatus','erpersonstatus','N','N','table_dropped','Person');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Person','erroles','erroles','Y','N','table_dropped','Person_erroles');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Person','givenname','givenname','Y','N','table_dropped','Person_givenname');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Person','sn','sn','Y','N','table_dropped','Person_sn');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('ProvisioningPolicy','DN','DN','N','N','table_dropped','ProvisioningPolicy');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('ProvisioningPolicy','erparent','erparent','N','N','table_dropped','ProvisioningPolicy');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('ProvisioningPolicy','erpolicyitemname','erpolicyitemname','N','N','table_dropped','ProvisioningPolicy');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('ProvisioningPolicy','erpolicymembership','erpolicymembership','Y','N','table_dropped','ProvPolicy_membership');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('ProvisioningPolicy','erpolicytarget','erpolicytarget','Y','N','table_dropped','ProvPolicy_targets');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Service','DN','DN','N','N','table_dropped','Service');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Service','erurl','erurl','N','N','table_dropped','Service');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Service','erparent','erparent','N','N','table_dropped','Service');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Service','erservicename','erservicename','N','N','table_dropped','Service');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Service','owner','owner','Y','N','table_dropped','Service_owner');
insert into enrole.entity_column (entity_name, column_name, attribute_name, multi_valued, implicitly_mapped, available_for_reporting, table_name) values ('Service','servicetype','servicetype','N','N','table_dropped','Service');


create table enrole.AUDIT_EVENT
(
	id NUMBER(20) NOT NULL,
	itim_event_category VARCHAR2(50) NOT NULL,
	entity_name VARCHAR2(1000),
	entity_dn VARCHAR2(1000),
	entity_type VARCHAR2(50),
	action VARCHAR2(25) NOT NULL,
	initiator_name VARCHAR2(1000),
	initiator_dn VARCHAR2(1000),
	container_name VARCHAR2(1000),
	container_dn VARCHAR2(1000),
	workflow_process_id NUMBER(20),
	result_summary VARCHAR2(25),
	timestamp VARCHAR2(50) NOT NULL,
	comments VARCHAR2(1000),
CONSTRAINT AUDIT_EVENT_PK PRIMARY KEY(id) USING
	INDEX
	TABLESPACE enrole_indexes
)
TABLESPACE enrole_data;

create table enrole.AUDIT_MGMT_PROVISIONING
(
	event_id NUMBER(20) NOT NULL,
	owner_name VARCHAR2(1000),
	owner_dn VARCHAR2(1000),
	service_name VARCHAR2(1000) NOT NULL,
	service_dn VARCHAR2(1000) NOT NULL,
	CONSTRAINT AUDIT_PROV_FK FOREIGN KEY(event_id)
	REFERENCES enrole.AUDIT_EVENT(id)
)
TABLESPACE enrole_data;


create table enrole.AUDIT_MGMT_TARGET
(
	event_id NUMBER(20) NOT NULL,
	target_entity_name VARCHAR2(1000),
	target_entity_dn VARCHAR2(1000),
	target_entity_type VARCHAR2(50),
	CONSTRAINT AUDIT_TARGET_FK FOREIGN KEY(event_id)
	REFERENCES enrole.AUDIT_EVENT(id)
)
TABLESPACE enrole_data;

create table enrole.AUDIT_MGMT_DELEGATE
(
	event_id NUMBER(20) NOT NULL,
	delegate_name VARCHAR2(1000),
	delegate_dn VARCHAR2(1000),
	delegate_start_time VARCHAR2(1000),
	delegate_end_time VARCHAR2(1000),
	CONSTRAINT AUDIT_DELEGATE_FK FOREIGN KEY(event_id)
	REFERENCES enrole.AUDIT_EVENT(id)
)
TABLESPACE enrole_data;

CREATE TABLE enrole.I18NMESSAGES
(
	PROFILE VARCHAR2(256),
	NAME VARCHAR2(256) NOT NULL,
	MESSAGEKEY VARCHAR2(256) NOT NULL,
	MESSAGE VARCHAR2(2000)
)
TABLESPACE enrole_data;

create table enrole.BULK_DATA_SERVICE
(
	id NUMBER(20) NOT NULL,
	starttime DATE,
	endtime DATE,
	mimetype VARCHAR(50),
	name VARCHAR(50),
	data LONG RAW,
	filesize Number(20),
	CONSTRAINT BULK_DATA_SERVICE_PK PRIMARY KEY(id)
)
TABLESPACE enrole_data;


create table enrole.BULK_DATA_STORE
(
	id NUMBER(20) NOT NULL,
	serviceid NUMBER(20) NOT NULL,
	xml LONG RAW,
	CONSTRAINT BULK_DATA_STORE_FK FOREIGN KEY (serviceid)
	REFERENCES BULK_DATA_SERVICE (id) ON DELETE CASCADE,
	CONSTRAINT BULK_DATA_STORE_PK PRIMARY KEY(id)
)
TABLESPACE enrole_data;

create table enrole.BULK_DATA_INDEX
(
	id NUMBER(20) NOT NULL,
	storeid NUMBER(20) NOT NULL,
	dataobjectid VARCHAR(10),
	CONSTRAINT BULK_DATA_INDEX_FK FOREIGN KEY (storeid)
	REFERENCES BULK_DATA_STORE (id) ON DELETE CASCADE,
	CONSTRAINT BULK_DATA_INDEX_PK PRIMARY KEY(id)
)
TABLESPACE enrole_data;

create table enrole.MIGRATION_STATUS
(
	ID NUMBER(20) NOT NULL,
	processCount INTEGER,
	processStatus VARCHAR(50),
	CONSTRAINT MIG_STATUS_PK PRIMARY KEY(ID)
)
TABLESPACE enrole_data;

CREATE VIEW enrole.ROOTPROCESSVIEW (ID, TYPE, REQUESTER) AS
(
	(SELECT ID, TYPE, REQUESTER
	FROM enrole.PROCESS
	WHERE  PROCESS.ROOT_PROCESS_ID  = 0
	OR PROCESS.ROOT_PROCESS_ID  = PROCESS.ID
	AND PROCESS.TYPE IN ('OP','AA','AC','UA','UC','UD','US','UR','UO','SR','OA','OC','AD',
						'AS','AR','AP','LO','AO','LS','LR','LP','MS','MR','MD','DA','DC','DD',
						'PA','PC','PD','SA','SC','SD','PS','EN','RC','PE','LC','CP','EP')
	)
	UNION
	(
	SELECT ID, 'AA', REQUESTER
	FROM enrole.PROCESS
	WHERE  PROCESS.ROOT_PROCESS_ID  = 0
	OR PROCESS.ROOT_PROCESS_ID  = PROCESS.ID
	AND PROCESS.TYPE IN ('OA')
	)

	UNION
	(
	SELECT ID, 'AC', REQUESTER
	FROM enrole.PROCESS
	WHERE  PROCESS.ROOT_PROCESS_ID  = 0
	OR PROCESS.ROOT_PROCESS_ID  = PROCESS.ID
	AND PROCESS.TYPE IN ('OC')
	)
);

CREATE VIEW enrole.SUBPROCESSVIEW (ROOT_PROCESS_ID, SUBMITTED,COMPLETED, SUBJECT_PROFILE, SUBJECT_SERVICE, SUBJECT,
	RESULT_SUMMARY, TYPE, REQUESTER)AS
(
	SELECT ROOT_PROCESS_ID, SUBMITTED,
		COMPLETED, SUBJECT_PROFILE, SUBJECT_SERVICE, SUBJECT,
		RESULT_SUMMARY, TYPE, REQUESTER
	FROM enrole.PROCESS
	WHERE PROCESS.TYPE IN ('OA','OC', 'AD', 'AS', 'AR', 'AP')
);

CREATE VIEW enrole.PENDING_APPROVAL
(PROCESSID, RESULT_SUMMARY)
AS
	(
		(
			SELECT PROCESS.ID, ACTIVITY.RESULT_SUMMARY
			FROM enrole.PROCESS, enrole.ACTIVITY, enrole.WORKITEM
			WHERE
				WORKITEM.PROCESS_ID = PROCESS.ID AND
				WORKITEM.ACTIVITY_ID = ACTIVITY.ID AND
				WORKITEM.LOCK_OWNER IS NULL
		)
		UNION
		(
			SELECT PROCESS.ID, 'LK'
			FROM enrole.PROCESS, enrole.ACTIVITY, enrole.WORKITEM
			WHERE
				WORKITEM.PROCESS_ID = PROCESS.ID AND
				WORKITEM.ACTIVITY_ID = ACTIVITY.ID AND
				WORKITEM.LOCK_OWNER IS NOT NULL
		)
);

CREATE TABLE SUMMARY_ORDER (
    RESULT_SUMMARY varchar2(2) NOT NULL,
    RESULT_SEVERITY NUMBER NOT NULL UNIQUE,
    CONSTRAINT Uni_ResultSummary PRIMARY KEY(RESULT_SUMMARY),
    CONSTRAINT Range_Value CHECK (RESULT_SEVERITY >= 0 AND RESULT_SEVERITY <= 100)
)
TABLESPACE enrole_data;

INSERT INTO SUMMARY_ORDER(RESULT_SUMMARY, RESULT_SEVERITY) VALUES ('SF', 100);
INSERT INTO SUMMARY_ORDER(RESULT_SUMMARY, RESULT_SEVERITY) VALUES ('SW', 99);
INSERT INTO SUMMARY_ORDER(RESULT_SUMMARY, RESULT_SEVERITY) VALUES ('ST', 98);
INSERT INTO SUMMARY_ORDER(RESULT_SUMMARY, RESULT_SEVERITY) VALUES ('AR', 97);
INSERT INTO SUMMARY_ORDER(RESULT_SUMMARY, RESULT_SEVERITY) VALUES ('PF', 96);
INSERT INTO SUMMARY_ORDER(RESULT_SUMMARY, RESULT_SEVERITY) VALUES ('SK', 95);
INSERT INTO SUMMARY_ORDER(RESULT_SUMMARY, RESULT_SEVERITY) VALUES ('ES', 94);
INSERT INTO SUMMARY_ORDER(RESULT_SUMMARY, RESULT_SEVERITY) VALUES ('AA', 93);
INSERT INTO SUMMARY_ORDER(RESULT_SUMMARY, RESULT_SEVERITY) VALUES ('RS', 92);
INSERT INTO SUMMARY_ORDER(RESULT_SUMMARY, RESULT_SEVERITY) VALUES ('PE', 1);
INSERT INTO SUMMARY_ORDER(RESULT_SUMMARY, RESULT_SEVERITY) VALUES ('SS', 0);
COMMIT;
